# -*- coding: utf-8 -*-
""" Cli commands
This file contains many commands for app managment
"""
import os
import subprocess

from pypkg_generator import package_generator


def install(dependency='', dev=False):
    """
    Installs a given python dependency,
    and adds them to Pipfile

    Parameters
    ----------
    dependency : str
       The python dependecy to install

    dev : bool
        If the python dependency is used during develoment
    """
    result = subprocess.run(['pipenv', 'install',
                             dependency, '--dev' if dev else ''],
                            cwd=os.getcwd())
    return result.returncode


def install_all(dev=False):
    """ Installs all packages from Pipfile
    (including dev dependencies if dev is provided)
    Parameters
    ----------
    dev : bool
        If dev is provided, install the dev dependencies
    """
    result = subprocess.run(['pipenv', 'install', '--dev' if dev else ''],
                            cwd=os.getcwd())
    return result.returncode


def uninstall(dependency=''):
    """ Uninstalls a python depdency
    Parameters
    ----------
    dependecy : str
        The python depdency to uninstall from the current virtual env
    """
    result = subprocess.run(['pipenv', 'uninstall', dependency],
                            cwd=os.getcwd())
    return result.returncode


def lock():
    """ Creates or updates the Pipfile.lock
    """
    result = subprocess.run(['pipenv', 'lock'], cwd=os.getcwd())
    return result.returncode


def shell():
    """ Spawns a shell within the virtualenv
    """
    result = subprocess.run(['pipenv', 'shell'], cwd=os.getcwd())
    return result.returncode


def build_source_wheel(version=3):
    """ Creates a source archive and a wheel for your package
    Parameters
    ----------
    version : int
        Defines the python version
    """
    if version == 3:
        result = subprocess.run(['python3', 'setup.py',
                                 'sdist', 'bdist_wheel'], cwd=os.getcwd())
    else:
        result = subprocess.run(['python', 'setup.py', 'sdist', 'bdist_wheel'],
                                cwd=os.getcwd())
    return result.returncode


def test_release():
    """ Uploads the package to test.pypi.org
    """
    result = subprocess.run(['twine', 'upload', '--repository-url',
                             'https://test.pypi.org/legacy/', 'dist/*'],
                            cwd=os.getcwd())
    return result.returncode


def release(remote=True):
    """ Uploads the package to pypi.org and pushes it to the git remote
    Parameters
    ----------
    remote : bool
        If remote is True, push the last commit to the git remote
    """
    result = subprocess.run(['twine', 'upload', 'dist/*'], cwd=os.getcwd())
    if remote:
        subprocess.run(['git', 'push', 'origin', 'master'], cwd=os.getcwd())
    return result.returncode


def install_pytest(pkg_dir=None):
    """ Installs pytest in a pinned version
    """
    result = subprocess.run(['pipenv', 'install', 'pytest==4.3.0',
                             '--dev'], cwd=pkg_dir)
    return result.returncode


def create_pkg(args):
    """ Creates a new python package from scratch
    Parameters
    ----------
    args : dict
        Contains the info to create the package
        according to needs of the user.
    """
    creator = package_generator.PackageGenerator(args)
    creator.call()



from setuptools import find_packages, setup

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='pybundler',
    version='0.2.1',
    description='Create or manage a python app or package',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Daniel Omar Vergara Pérez',
    author_email='daniel.omar.vergara@gmail.com',
    url='https://github.com/dany2691/pybundler',
    packages=find_packages(exclude=('tests',)),
    install_requires=[
        'click>=7.0.0',
        'colored>=1.3.93',
        'pypkg-generator>=0.4.1',
        'pipenv>=2018.11.26',
        'twine>=1.13.0'
    ],
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    entry_points={
        'console_scripts': [
            'pybundler=pybundler.__main__:main'
        ]
    }
)



# -*- coding: utf-8 -*-
"""Main Function and entry point of the package

This script requires that `click` be installed within the Python
environment you are running this script in.

This file contains the following functions:

    * conduct_option - Returns the conduct file option
    * license_option - Returns the license file option
    * pipfile_option - Returns the pipfile file option
    * main - the main function of the script

"""
import os

import click
from colored import fg, attr

from pybundler import cli


@click.command()
@click.option('--install-all', is_flag=True,
              help='Installs all packages from Pipfile')
@click.option('--install', '-i', default='',
              help='Install a given package or the content of the Pipfile')
@click.option('--uninstall', '-u',
              help='uninstalls a given dependecy')
@click.option('--dev', '-d', is_flag=True,
              help='If it is True, install dependency in dev section')
@click.option('--lock', is_flag=True,
              help='Creates or updates the Pipfile.lock')
@click.option('--shell', '-s', is_flag=True,
              help='Spawns a shell within the virtualenv')
@click.option('--build-wheel', '-b', is_flag=True,
              help='Creates a source archive and a wheel for your package')
@click.option('--test-release', '-t', is_flag=True,
              help='Uploads the package to test.pypi.org')
@click.option('--release', '-r', is_flag='True',
              help='Uploads the package to pypi.org and'
                   'pushes it to the git remote')
@click.option('--package', '-p', is_flag=True,
              help='Creates a new python package from scratch')
def main(install_all, install, uninstall, dev, lock, shell,
         build_wheel, test_release, release, package):
    """Executes the entire program

    Parameters \n
    ----------- \n
    install_all : bool\n
        Installs all packages from Pipfile\n
    install : str\n
        The given dependency that is going to be installed\n
    uninstall: str\n
        Uninstalls the given dependency and removes it from Pipfile\n
    dev : bool\n
        Installs the given dependency as a develoment dependency\n
    lock : bool\n
        Generates or updates Pipfile.lock\n
    shell : bool\n
         Spawns a shell within the virtualenv\n
    build_wheel : bool\n
        Creates a source archive and a wheel for your package\n
    test_release : bool\n
        Uploads the package to test.pypi.org\n
    release : bool\n
        Uploads the package to pypi.org and pushes it to the git remote\n
    package : bool\n
        Creates a new python package from scratch\n
    """
    if install_all:
        cli.install_all(dev)

    if install:
        cli.install(install, dev)

    if uninstall:
        cli.uninstall(uninstall)

    if lock:
        cli.lock()

    if shell:
        cli.shell()

    if build_wheel:
        cli.build_source_wheel()

    if test_release:
        cli.test_release()

    if release:
        cli.release()

    if package:
        pkg_name = click.prompt(enter_name())
        path = click.prompt(enter_path(), default='')
        license_file = click.confirm(license_option())
        conduct_file = click.confirm(conduct_option())
        pipfile = click.confirm(pipfile_option())
        pytest = click.confirm(install_pytest_confirmation())
        args = {'name': pkg_name, 'path': path, 'tests': True,
                'license': license_file, 'code_of_conduct': conduct_file,
                'pipfile': pipfile}
        cli.create_pkg(args)

        if pytest:
            cli.install_pytest(os.path.join(path, pkg_name))


def conduct_option():
    """Returns the conduct file option"""
    return f'{fg(2)} Do you want to include a code of conduct file? {attr(0)}'


def license_option():
    """Returns the license file option"""
    return f'{fg(2)} Do you want to include a license file? {attr(0)}'


def pipfile_option():
    """Returns the pipfile file option"""
    return f'{fg(2)} Do you want to include a Pipfile file? {attr(0)}'


def enter_name():
    """Ask for the name of the new package"""
    return f'{fg(2)} Please, enter the nanme of the package: {attr(0)}'


def enter_path():
    """Ask for the path of the new project"""
    return f'{fg(2)}Enter the path of the project (default: current dir){attr(0)}'


def install_pytest_confirmation():
    """Ask if pytest should be installed"""
    return f'{fg(2)} Do you want to install pytest? {attr(0)}'


if __name__ == '__main__':
    main()
